/*
* Jalview - A Sequence Alignment Editor and Viewer
* Copyright (C) 2005 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
*/
package jalview.io;

import jalview.datamodel.*;

import java.io.*;

import java.util.*;


/**
 * DOCUMENT ME!
 *
 * @author $author$
 * @version $Revision$
 */
public abstract class AlignFile extends FileParse
{
    int noSeqs = 0;
    int maxLength = 0;
    Vector seqs;
    Vector headers;
    long start;
    long end;
    boolean dbPrefix = true;
    boolean jvSuffix = true;

    /**
     * Creates a new AlignFile object.
     */
    public AlignFile()
    {
    }

    /**
     * Creates a new AlignFile object.
     *
     * @param inStr DOCUMENT ME!
     */
    public AlignFile(String inStr)
    {
        initData();
        System.out.println("is this ever called??");

        try
        {
            parse();
        }
        catch (Exception ex)
        {
        }
    }

    /**
     * Constructor which parses the data from a file of some specified type.
     * @param inFile Filename to read from.
     * @param type   What type of file to read from (File, URL)
     */
    public AlignFile(String inFile, String type) throws IOException
    {
        super(inFile, type);

        initData();

        parse();
    }

    /**
     * Return the seqs Vector
     */
    public Vector getSeqs()
    {
        return seqs;
    }

    /**
     * Return the Sequences in the seqs Vector as an array of Sequences
     */
    public SequenceI[] getSeqsAsArray()
    {
        SequenceI[] s = new SequenceI[seqs.size()];

        for (int i = 0; i < seqs.size(); i++)
        {
            s[i] = (SequenceI) seqs.elementAt(i);
        }

        return s;
    }

    /**
     * Initialise objects to store sequence data in.
     */
    protected void initData()
    {
        seqs = new Vector();
        headers = new Vector();
    }

    /**
     * DOCUMENT ME!
     *
     * @param s DOCUMENT ME!
     */
    protected void setSeqs(SequenceI[] s)
    {
        seqs = new Vector();

        for (int i = 0; i < s.length; i++)
        {
            seqs.addElement(s[i]);
        }
    }

    // Checks whether sequence is valid aa characters
    protected boolean isValidProteinSequence(String sequence)
    {
        for (int i = 0; i < sequence.length(); i++)
            if (!jalview.schemes.ResidueProperties.aaHash.containsKey(
                        String.valueOf(sequence.charAt(i))))
            {
                return false;
            }

        return true;
    }

    /**
     * This method must be implemented to parse the contents of the file.
     */
    public abstract void parse() throws IOException;

    /**
     * Print out in alignment file format the Sequences in the seqs Vector.
     */
    public abstract String print();

    public void addDBPrefix(boolean b)
    {
      dbPrefix = b;
    }

    public void addJVSuffix(boolean b)
    {
      jvSuffix = b;
    }

    /**
     * A general parser for ids. Will look for dbrefs in
     * Uniprot format source|id
     * And also Jalview /start-end
     *
     * @String id Id to be parsed
     */
    Sequence parseId(String id)
    {
      id = id.trim();
      Sequence seq = new Sequence("","");
      int space = id.indexOf(" ");
      if(space>-1)
      {
        seq.setDescription(id.substring(space+1));
        id = id.substring(0, space);
      }

      // Read in any DB refs first
      StringTokenizer st;
      st = new StringTokenizer(id, "|");

      while (st.countTokens()>1)
      {
        String a = st.nextToken();
        String b = st.nextToken();
        seq.addDBRef( new DBRefEntry( a, "0", b));
      }

      if(st.hasMoreTokens())
        id = st.nextToken();


      // Remove /start-end from sequence
      if (id.indexOf("/") > 0)
      {
        st = new StringTokenizer(id, "/");
        String limits=null;
        try{
          if (st.countTokens() == 2)
          {
            id = st.nextToken();

            limits = st.nextToken();

            st = new StringTokenizer(limits, "-");

            if (st.countTokens() == 2)
            {
              seq.setStart(Integer.valueOf(st.nextToken()).intValue());
              seq.setEnd(Integer.valueOf(st.nextToken()).intValue());
            }
          }
        }catch(NumberFormatException ex)
        {
            // Problem parsing sequence limits. Just add it back to the
          // Id so we dont lose this info
          id += "/" + limits;
        }
      }
      seq.setName(id);
      return seq;
    }

    /**
     * Creates the output id.
     * Adds prefix Uniprot format source|id
     * And suffix Jalview /start-end
     *
     * @String id Id to be parsed
     */
    String printId(SequenceI seq)
    {
      StringBuffer result = new StringBuffer();
      if(dbPrefix && seq.getDBRef()!=null)
      {
        Vector dbrefs = seq.getDBRef();
        for(int i=0; i<dbrefs.size(); i++)
        {
          DBRefEntry entry = (DBRefEntry)dbrefs.elementAt(i);
          result.append(entry.getSource()+"|"+entry.getAccessionId()+"|");
        }
      }

      result.append(seq.getName());

      if(jvSuffix)
      {
        result.append("/"+seq.getStart()+"-"+seq.getEnd());
      }

      return result.toString();
    }

}
